function idl=findidl(ref,numofpulses,pulserate,pulsewidth,resptype,electconfig,N,method,percnt);
%
% idl=findidl(ref,numofpulses,pulserate,pulsewidth,resptype,electconfig,N,method,percnt);
%
% FINDIDL Finds the IDL corresponding to a performance percentage of input
% variable percnt
%
% INPUT:
%
% ref:
%   is the reference stimulus intensity (in dB re. 1 microA) for method = 'idl'
%   is ignored for method = 'thr'
%
% numofpulses is the number of pulses in the pulse train
%
% pulserate is the rate of stimulation in pulses/s
%   note that for pulserate <= 100 pulse/s the singlepulse model is used,
%   otherwise the pulsetrain model is used
%
% pulsewidth is the phase duration in microseconds/phase
%
% resptype is the fiber I/O function type:
%   'stp' = deterministic (step-function) model
%   '3_p' = 3-piece linear model
%   'erf' = stochastic (error-function) model
%
% electconfig is the electrode configuration:
%   'mp' = monopolar; 'bp' = bipolar
%
% N is the number of fibers
%
% method sets the reference stimulus level:
%   'thr' sets reference level to 0 microA, i.e., finds threshold
%   'idl' sets reference level to input variable ref
%
% percnt is the desired performance percentage
%
% OUTPUT:
%
% idl is the stimulus level producing a performance percentage equal to the
% input variable percnt
%
% ex. 1: idl = findidl([],1,1,100,'erf','mp',1e4,'thr',70.71)
%   finds the threshold stimulus level (corresponding to a 70.71% performance)
%   for a single pulse of phase duration 100 microseconds/phase, the stochastic
%   model, monopolar stimulation and 10,000 fibers.
%
% ex. 2: idl = findidl(45,1,1,100,'erf','bp',1e4,'idl',80)
%   finds the idl (corresponding to a 80% performance) for a reference stimulus
%   level of 45 dB re. 1 microA for a single pulse of phase duration 100
%   microseconds/phase, the stochastic model, bipolar stimulation and
%   10,000 fibers.
%
% Usage Agreement:  Any publication of results obtained using this model
%                   should cite-
%
%       [1] Bruce, I. C., White, M. W., Irlicht, L. S., O'Leary, S. J., Dynes,
%           S., Javel, E., Clark, G. M. (1999) "A stochastic model of the
%           electrically stimulated auditory nerve: Single-pulse response,"
%           IEEE Transactions on Biomedical Engineering, 46(6):617-629.
%
%   and
%
%       [2] Bruce, I. C., White, M. W., Irlicht, L. S., O'Leary, S. J., Clark,
%           G. M. (1999) "The effects of stochastic neural activity in a model
%           predicting intensity perception with cochlear implants: Low-rate
%           stimulation," IEEE Transactions on Biomedical Engineering,
%           46(12):1393-1404.
%
% If using pulse rates >= 200 pps, you should also cite:
%
%       [3] Bruce, I. C., Irlicht, L. S., White, M. W., O'Leary, S. J., Dynes,
%           S., Javel, E., Clark, G. M. (1999) "A stochastic model of the
%           electrically stimulated auditory nerve: Pulse-train response,"
%           IEEE Transactions on Biomedical Engineering, 46(6):630-637.
%
%   and
%
%       [4] Bruce, I. C., Irlicht, L. S., White, M. W., O'Leary, S. J., Clark,
%           G. M. (2000) "Renewal-process approximation of a stochastic
%           threshold model for electrical neural stimulation," Journal of
%           Computational Neuroscience, 9:119-132.

% Copyright (c) 1996-2001 Ian Bruce
% Created by Ian Bruce, 1996
% Released for public distribution as version 1.2, 2001

steps = 35;   % Number of steps in range of stimulus intensities over which to search: Reduce if memory runs out
minstim  = 0;   % Minimum of range of stimulus intensities over which to search for threshold measure
maxstim  = 100; % Maximum of range of stimulus intensities over which to search

if method=='idl'
    stepsize = (maxstim-ref)/steps;
    stim = ref:stepsize:maxstim;
elseif method=='thr'
    stepsize = (maxstim-minstim)/steps;
    stim = minstim:stepsize:maxstim;
end

% Get psychometric function
correct = psychomet(stim,numofpulses,pulserate,pulsewidth,resptype,electconfig,N,method);

% Find the part of the psychometric function that goes from 50% to 100%
while (resptype~='stp'&(sum(correct<51)>1|sum(correct>99)>1))|(resptype=='stp'&stepsize>1e-4)
    
    estlo = stim(max(find(correct<51)));
    esthi = stim(min(find(correct>99)));
    
    stepsize = (esthi-estlo)/steps;
    
    if method=='thr'|estlo==ref
        stim = estlo:stepsize:esthi;
    else
        stim = [ref estlo:stepsize/40:esthi];
    end   
    
    correct = psychomet(stim,numofpulses,pulserate,pulsewidth,resptype,electconfig,N,method);
    
end

% Interpolate psychometric function to find stimulus intensity corresponding to performance
% of input variable percnt
if resptype=='stp'
    idl = mean(stim([max(find(correct<100)) min(find(correct>50))]));
else     
    idl = interp1(correct,stim,percnt);
end
