function [percentcorrect, cnt, cntstd] = psychomet(stim,numofpulses,pulserate,pulsewidth,resptype,electconfig,N,method)
%
% [percentcorrect, cnt, cntstd] = psychomet(stim,numofpulses,pulserate,pulsewidth,resptype,electconfig,N,method)
%
% PSYCHOMET Calculates the psychometric function for a 2IFC intensity difference
% limen procedure using TOTALAN to give the neural count.  The decision rule is
% simply that the stimulus producing the higher count is chosen as the louder.
% If the counts are equal, then a guess is made.
%
% INPUT:
%
% stim is a vector of stimulus intensities in dB re. 1 microA
%
% numofpulses is the number of pulses in the pulse train
%
% pulserate is the rate of stimulation in pulses/s
%   note that for pulserate <= 100 pulse/s the singlepulse model is used,
%   otherwise the pulsetrain model is used
%
% pulsewidth is the phase duration in microseconds/phase
%
% resptype is the fiber I/O function type:
%   'stp' = deterministic (step-function) model
%   '3_p' = 3-piece linear model
%   'erf' = stochastic (error-function) model
%
% electconfig is the electrode configuration:
%   'mp' = monopolar; 'bp' = bipolar
%
% N is the number of fibers
%
% method sets the reference stimulus level:
%   'thr' sets reference level to 0 microA, i.e., finds threshold
%   'idl' sets reference level to the first element of the input vector 'stim'
%
% OUTPUT:
%
% percentcorrect is the predicted performance for the 2IFC procedure
%
% cnt is the mean number of discharges for the total Auditory Nerve (AN)
% cntstd is the standard deviation in number of discharges for the total AN
%
% ex. 1:
% [percentcorrect,cnt,cntstd] = psychomet(20:2:60,1,1,100,'erf','mp',1e4,'thr')
%   gives the predicted performance for threshold determination at stimulus
%   intensities from 20 dB to 60 dB re. 1 microA in 2 dB steps, for a single
%   pulse of phase duration 100 microseconds/phase, the stochastic model,
%   monopolar stimulation and 10,000 fibers.
%
% ex. 2:
% [percentcorrect,cnt,cntstd] = psychomet(40:2:80,1,1,100,'stp','bp',1e4,'idl')
%   gives the predicted performance for IDL determination relative to 40 dB
%   re. 1 microA at stimulus intensities from 40 dB to 80 dB re. 1 microA in
%   2 dB steps, for a single pulse of phase duration 100 microseconds/phase,
%   the deterministic model, bipolar stimulation and 10,000 fibers.
%
% Usage Agreement:  Any publication of results obtained using this model
%                   should cite-
%
%       [1] Bruce, I. C., White, M. W., Irlicht, L. S., O'Leary, S. J., Dynes,
%           S., Javel, E., Clark, G. M. (1999) "A stochastic model of the
%           electrically stimulated auditory nerve: Single-pulse response,"
%           IEEE Transactions on Biomedical Engineering, 46(6):617-629.
%
%   and
%
%       [2] Bruce, I. C., White, M. W., Irlicht, L. S., O'Leary, S. J., Clark,
%           G. M. (1999) "The effects of stochastic neural activity in a model
%           predicting intensity perception with cochlear implants: Low-rate
%           stimulation," IEEE Transactions on Biomedical Engineering,
%           46(12):1393-1404.
%
% If using pulse rates >= 200 pps, you should also cite:
%
%       [3] Bruce, I. C., Irlicht, L. S., White, M. W., O'Leary, S. J., Dynes,
%           S., Javel, E., Clark, G. M. (1999) "A stochastic model of the
%           electrically stimulated auditory nerve: Pulse-train response,"
%           IEEE Transactions on Biomedical Engineering, 46(6):630-637.
%
%   and
%
%       [4] Bruce, I. C., Irlicht, L. S., White, M. W., O'Leary, S. J., Clark,
%           G. M. (2000) "Renewal-process approximation of a stochastic
%           threshold model for electrical neural stimulation," Journal of
%           Computational Neuroscience, 9:119-132.

% Copyright (c) 1996-2001 Ian Bruce
% Created by Ian Bruce, 1996
% Released for public distribution as version 1.2, 2001

if any(diff(stim)<=0)
    error('Stimulus vector must consist of ascending, unique values')
end

if method == 'thr'
    stim=[-Inf stim]; % For threshold measurement, stimulus intensity = 0 microA = -Inf dB re. 1 microA
elseif method == 'idl'
else
    error('Unknown value of input variable method')
end

% Initialize matrices
cnt=zeros(size(stim));
cntstd=zeros(size(stim));
percentcorrect = zeros(size(stim));

% Calculate cochlear response

[tmpa, tmpb, cnt, cntstd] = totalan(stim,numofpulses,pulserate,pulsewidth,resptype,electconfig,N);

clear tmpa tmpb % Use only summed AN response, clear memory of single-fiber discharge probabilities

if (resptype(1:3) == 'stp')
    % For deterministic model:
    %   spike counts equal to spike count for reference intensity     =>  50% performance
    %   spike counts greater than spike count for reference intensity => 100% performance
    percentcorrect = (sign(cnt-cnt(:,1)*ones(1,size(cnt,2)))==1).*100+(sign(cnt-cnt(:,1)*ones(1,size(cnt,2)))==0).*50;
else
    
    swppnt=15; % Point where distribution changes from Poisson to Gaussian (see p. 1396 of [2])
    
    for lp=1:length(cnt)
        if cnt(1)+3*cntstd(1)<cnt(lp)-3*cntstd(lp)
            % If mean spike counts separated by more than 3 standard deviations,
            %   assume 100% performance
            percentcorrect(lp) = 100;
        else
            % Eqn. (3) of [2]
            PrGr = sum(jointpr(0:ceil(cnt(lp)+4*cntstd(lp)),N*numofpulses,cnt(1),cntstd(1),cnt(lp),cntstd(lp),'gr',swppnt));
            PrEq = sum(jointpr(0:ceil(cnt(lp)+4*cntstd(lp)),N*numofpulses,cnt(1),cntstd(1),cnt(lp),cntstd(lp),'eq',swppnt));
            percentcorrect(lp) = 100*PrGr+50*PrEq;           
        end       
    end
end

if method=='thr'
    % Throw out first value of percentcorrect (always 50%) so elements of percentcorrect vector
    % line up with corresponding elements of stim vector
    percentcorrect = percentcorrect(2:length(percentcorrect));
end

return;

function p=jointpr(x,N,mu1,sig1,mu2,sig2,PrType,swppnt)
% Calculate joint probability for two distrubtions
if mu2<swppnt
    if PrType=='gr'
        % Calculate inner value of product for X1 > X2
        for lp=1:length(x)
            inner(lp)= sum(poisson(x(lp)+1:ceil(poisscon(mu2)+2),mu2)); % See Eqn. (3) of [2]
        end
    else
        % Calculate inner value of product for X1 = X2
        inner = poisson(x,mu2); % See Eqn. (3) of [2]
    end
else
    if PrType=='gr'
        % Calculate inner value of product for X1 > X2
        for lp=1:length(x)
            inner(lp) = sum(gaussian(x(lp)+1:ceil(mu2+4*sig2),mu2,sig2)); % See Eqn. (3) of [2]
        end
    else
        % Calculate inner value of product for X1 = X2
        inner = gaussian(x,mu2,sig2); % See Eqn. (3) of [2]
    end
end

if mu1<swppnt % Calculate outer value of product
    outer = poisson(x,mu1);
else
    outer = gaussian(x,mu1,sig1);
end

p = outer.*inner; % See Eqn. (3) of [2]

return;

function f=poisson(x,lam)

f=exp(-lam).*lam.^x./gamma(x+1); % Eqn. (4) of [2]

return;

function gauss=gaussian(x,mu,sig)

gauss=1/sqrt(2*pi)/sig*exp(-(x-mu).^2/2/sig^2); % Eqn. (5) of [2]

return;

function y=poisscon(lambda)

% Calculates the lowest value of x for which the Poisson distribution
% is close enough to zero to be used with numerical sum or integration.
%
% The point of convergence is taken to be the value of x one greater that where
% the cumulative distribution is equal to 1.000 in Table A.2 in "Probability and
% Statistics for Engineering and the Sciences", 2nd ed., Jay L Devore
%
% Polynomial regression constants for point of convergence of poisson to 0
% vs lambda
%
% b[0]	3.3316267734
% b[1]	3.5066392181
% b[2]	-.3320104691
% b[3]	.0218473886
% b[4]	-4.9038663448e-4

y=3.3316267734+3.5066392181*lambda-0.3320104691*lambda^2+0.0218473886*lambda^3-4.9038663448e-4*lambda^4;

return;
