function disprob=singlepulse(I,properties)

% disprob=singlepulse(I,properties)
%
% INPUT:
%
% I is the stimulus intensity in units of microA, NOT dB
%
% properties is a structure containing the model parameters:
%
%   properties.resptype is the fiber I/O function type:
%       'stp' = deterministic (step-function) model
%       '3_p' = 3-piece linear model
%       'erf' = stochastic (error-function) model
%
%   properties.thrsh is the matrix of fiber thresholds
%
%   properties.noisestd is the matrix of fiber noise standard deviations
% 
% OUTPUT:
%
% disprob the discharge probability (mean discharge rate) per pulse
%
% Usage Agreement:  Any publication of results obtained using this model
%                   should cite-
%
%       [1] Bruce, I. C., White, M. W., Irlicht, L. S., O'Leary, S. J., Dynes,
%           S., Javel, E., Clark, G. M. (1999) "A stochastic model of the
%           electrically stimulated auditory nerve: Single-pulse response,"
%           IEEE Transactions on Biomedical Engineering, 46(6):617-629.

% Copyright (c) 1996-2001 Ian Bruce
% Created by Ian Bruce, 1996
% Released for public distribution as version 1.1, 1999
% Version 1.2 released in 2001 with updated citation details

resptype = properties.resptype;

if resptype == 'stp'
    
    thrsh = properties.thrsh;
    
    disprob = (I<thrsh).*0+(I>=thrsh).*1; % Eqn. (2) of [1]
    
elseif resptype == '3_p'
    
    disprob = zeros(size(I));
    
    thrsh = properties.thrsh;
    noisestd = properties.noisestd/1.3820;
    
    A = thrsh-sqrt(3).*noisestd;
    B = thrsh+sqrt(3).*noisestd;
    
    disprob = (I<A).*0+(I>=A).*(I<=B).*(1./(B-A).*I-A./(B-A))+(I>B).*1;
    
elseif resptype=='erf'
    
    disprob = zeros(size(I));
    
    thrsh = properties.thrsh;
    noisestd = properties.noisestd;
    
    disprob = 1/2*(erf((I-thrsh)/sqrt(2)./noisestd)+1); % Eqn. (3) of [1]
    
else
    error(['The fiber I/O function ' resptype ' is not yet supported'])
end
